__version__ = "2.2.0"

from . import (article, beikeisland, collection, island, notebook, objects,
               rank, user)


def future():
    print("回望洪荒，历史苍茫；\n只身向前，重铸星光。\n赴梦万里，道阻且长；\n今日萤火，明日华章。")
